@@ -43,7 +43,7 @@
 // as explicitly stated in "COPYING".
 
 //
-// $Id: dsp.cpp,v 1.4 2002/05/30 21:18:58 dyfet Exp $
+// $Id: dsp.cpp,v 1.6 2003/04/02 23:23:14 dyfet Exp $
 //
 
 #include "driver.h"
@@ -69,6 +69,10 @@ bool AculabTrunk::Join(AculabTrunk *trunk)
 	freeDSP();
 	trunk->freeDSP();
 
+	memset(&detail_xparms, 0, sizeof(detail_xparms));
+	memset(&connection, 0, sizeof(connection));
+	memset(&trunk->detail_xparms, 0, sizeof(trunk->detail_xparms));
+
 	detail_xparms.valid=0;
 	detail_xparms.handle=handle;
 	detail_xparms.timeout=500;
@@ -121,7 +125,11 @@ bool AculabTrunk::Join(AculabTrunk *trunk)
 		 * chosen by this application to route data from card 1
 		 * to card 0) to network port timeslot.
 		 */
+#ifdef MVIP_BASED
 		connection.ist=mvip_stream;
+#else
+		connection.ist = mvip_stream + MVIP_FD_STREAMS;
+#endif
 		connection.its=mvip_slot;
 		connection.mode=CONNECT_MODE;
 		connection.ost=detail_xparms.stream;
@@ -138,7 +146,11 @@ bool AculabTrunk::Join(AculabTrunk *trunk)
 		connection.ist=trunk->detail_xparms.stream;
 		connection.its=trunk->detail_xparms.ts;
 		connection.mode=CONNECT_MODE;
+#ifdef MVIP_BASED
 		connection.ost=mvip_stream+8;
+#else
+		connection.ost = mvip_stream + MVIP_FD_STREAMS;
+#endif
 		connection.ots=mvip_slot;
 		rc=sw_set_output(switch_driver1,&connection);
 		slog(Slog::levelDebug)<<name<<": join: connect card 1->mvip, rc="<<rc<<endl;
@@ -148,7 +160,11 @@ bool AculabTrunk::Join(AculabTrunk *trunk)
 		 * chosen by this application to route data from card 0
 		 * to card 1) to network port timeslot.
 		 */
+#ifdef MVIP_BASED
 		connection.ist=mvip_stream+8;
+#else
+		connection.ist = mvip_stream;
+#endif
 		connection.its=mvip_slot;
 		connection.mode=CONNECT_MODE;
 		connection.ost=trunk->detail_xparms.stream;
@@ -195,7 +211,6 @@ void AculabTrunk::Part(void)
 {
 	OUTPUT_PARMS	connection;
 	TrunkEvent event;
-	AculabTrunk *trunk;
 	int switch_driver0;
 	int switch_driver1;
 	int rc;
@@ -219,7 +234,11 @@ slog(Slog::levelDebug) <<name<<": parting trunks"<<endl;
 		rc=sw_set_output(switch_driver0,&connection);
 slog(Slog::levelDebug) <<name<<":disable mvip stream on card 0, rc="<<rc<<endl;
 
+#ifdef MVIP_BASED
 		connection.ost=mvip_stream+8;
+#else
+		connection.ost = mvip_stream + MVIP_FD_STREAMS;
+#endif
 		connection.ots=mvip_slot;
 		connection.mode=DISABLE_MODE;
 		rc=sw_set_output(switch_driver1,&connection);
@@ -227,7 +246,8 @@ slog(Slog::levelDebug) <<name<<":disable mvip stream on card 1,rc="<<rc<<endl;
 
 		aculabivr.freeMVIPslot(mvip_stream,mvip_slot);
 	}
-	else {
+	
+//	else {
 		connection.ost=detail_xparms.stream;
 		connection.ots=detail_xparms.ts;
 		connection.mode=DISABLE_MODE;
@@ -239,13 +259,13 @@ slog(Slog::levelDebug) <<name<<":disable mvip stream on call 0,rc="<<rc<<endl;
 		connection.mode=DISABLE_MODE;
 		rc=sw_set_output(switch_driver1,&connection);
 slog(Slog::levelDebug) <<name<<":disable mvip stream on call 1,rc="<<rc<<endl;
-	}
+//	}
 
         join->join = NULL;
         event.id = TRUNK_PART_TRUNKS;
         join->postEvent(&event);
         join = NULL;
-}	
+}
 
 /*
  * Allocate dsp resources of a given type and connect
@@ -315,7 +335,7 @@ slog(Slog::levelDebug)<<name<<": connecting DSP channel to prosody"<<endl;
 	chRgEvParms.event_type   = kSMEventTypeRecog;
 	chRgEvParms.issue_events = kSMAnyChannelEvent;
 	sm_channel_set_event(&chRgEvParms);
-                
+
 	/*
 	 * Also have driver notify fire off a 'any channel write' event
 	 * if we need to write data (playing audio).
@@ -386,9 +406,19 @@ bool AculabTrunk::connectToDSP()
 {
 	struct sm_channel_info_parms sm_channel_info_parms;
 	struct output_parms sw_output_parms;
+	int    rc;
 	mvip_stream = -1;
 	mvip_slot = -1;
 
+
+	/* Aculab's API may get extended in the future - if we do
+	 * not set the structures to zero before use, then spurious
+	 * values may be interpreted as meaning something at a later
+	 * date.
+	 */
+	memset(&sm_channel_info_parms, 0, sizeof(sm_channel_info_parms));
+	memset(&sw_output_parms, 0, sizeof(sw_output_parms));
+
 	sm_channel_info_parms.channel=dspChannel;
 	sm_channel_info(&sm_channel_info_parms);
 
@@ -411,7 +441,12 @@ bool AculabTrunk::connectToDSP()
 		 * Channel is hosted on card whose channels are terminated
 		 * on Digital Switch Matrix (BR4/BR8) and channel is to be
 		 * switched to network port on same card.
-		 */ 
+		 *
+		 * Most systems are now based on PCI cards.  If PCI cards
+		 * are used, then the speech processing DSP is usually located
+		 * on the same card as the network termination (Prosody PCI & cPCI)
+		 * so this path is actually the most likely...
+		 */
 		sw_output_parms.ist  = sm_channel_info_parms.ost;
 		sw_output_parms.its  = sm_channel_info_parms.ots;
 		sw_output_parms.ost  = call_port_2_stream(port);
@@ -422,25 +457,26 @@ bool AculabTrunk::connectToDSP()
 			slog(Slog::levelError) <<name<<": driver not installed to assign chan ts to expansion bus!"<<endl;
 			return false;
 		}
-	
+
 		sw_output_parms.ist  = call_port_2_stream(port);
 		sw_output_parms.its  = ts;
 		sw_output_parms.ost  = sm_channel_info_parms.ist;
 		sw_output_parms.ots  = sm_channel_info_parms.its;
 		sw_output_parms.mode = CONNECT_MODE;
-		
+
 		if (sw_set_output(call_port_2_swdrvr(port),&sw_output_parms) != 0) {
 			slog(Slog::levelError) <<name<<": driver not installed to assign chan ts to expansion bus!"<<endl;
 			return false;
 		}
 	}
 	else {
+#ifdef MIVP_BASED
 		/*
 		 * Channel is hosted on card whose channels are terminated on
 		 * Digital Switch Matrix (BR4/BR8) and channel is to be
 		 * switched to network port on a different card, thus need
 		 * to make connections between two cards on MVIP bus.
-		 */ 
+		 */
 
 		/*
 		 * Get the first available full-duplex MVIP stream and timeslot
@@ -468,7 +504,7 @@ bool AculabTrunk::connectToDSP()
 		sw_output_parms.ots=mvip_slot;
 		sw_output_parms.mode=CONNECT_MODE;
 		sw_set_output(call_port_2_swdrvr(port), &sw_output_parms);
-		
+
 		/*
 		 * This case is used if Prosody is on a BR card but the call
 		 * is on an E1 card.
@@ -489,8 +525,88 @@ bool AculabTrunk::connectToDSP()
 		sw_output_parms.ots=sm_channel_info_parms.its;
 		sw_output_parms.mode=CONNECT_MODE;
 		sw_set_output(sm_channel_info_parms.card,&sw_output_parms);
-	}
+#else
+	/* MVIP is usually associated with ISA cards.
+	 * PCI (cPCI) cards support the ECTF H.100 (H.110) CT Bus.
+	 * H.100 is significantly wider than MVIP, so is less likely
+	 * to become the limiting factor in how many resources can
+	 * be hosted in a chassis.
+	 */
 
+		mvip_stream = mvip_slot = 0;
+		if(!aculabivr.getMVIPslot(&mvip_stream, &mvip_slot))
+		{
+			slog(Slog::levelError) << name << ": no free H.100 slots!" << endl;
+			return false;
+		}
+
+	/* H.100 does not have the same concept of direction as MVIP.
+	 * It is just a set of connections...
+	 */
+
+	/* connect from call -> H.100 */
+		/* NOTE: we should find the stream associated with the call
+		 * using the call_details function.  It is possible (VoIP)
+		 * that the port may have multiple streams associated with it!
+		 */
+		sw_output_parms.ist = call_port_2_stream(port);
+		sw_output_parms.its = ts;
+		sw_output_parms.ost = mvip_stream;
+		sw_output_parms.ots = mvip_slot;
+
+		sw_output_parms.mode = CONNECT_MODE;
+
+		rc = sw_set_output(call_port_2_swdrvr(port), &sw_output_parms);
+		if(rc)
+		{
+			slog(Slog::levelError) << name << ": sw_set_output failed (" << rc << ")" << endl;
+			/* probably free up the MVIP slot, but we're pretty dead at this point! */
+			return false;
+		}
+
+	/* connect from H.100 to the call */
+		sw_output_parms.ist = mvip_stream + MVIP_FD_STREAMS;
+		sw_output_parms.its = mvip_slot;
+		sw_output_parms.ost = call_port_2_stream(port);
+		sw_output_parms.ots = ts;
+
+		sw_output_parms.mode = CONNECT_MODE;
+		rc = sw_set_output(call_port_2_swdrvr(port), &sw_output_parms);
+		if(rc)
+		{
+			slog(Slog::levelError) << name << ": sw_set_output failed (" << rc << ")" << endl;
+			return false;
+		}
+
+	/* connect from the DSP resource to H.100 */
+		sw_output_parms.ist = sm_channel_info_parms.ost;
+		sw_output_parms.its = sm_channel_info_parms.ots;
+		sw_output_parms.ost = mvip_stream + MVIP_FD_STREAMS;
+		sw_output_parms.ots = mvip_slot;
+
+		sw_output_parms.mode = CONNECT_MODE;
+		rc = sw_set_output(sm_channel_info_parms.card, &sw_output_parms);
+		if(rc)
+		{
+			slog(Slog::levelError) << name << " : sw_set_output failed (" << rc << ")" << endl;
+			return false;
+		}
+
+	/* connect from H.100 to the DSP resource */
+		sw_output_parms.ist = mvip_stream;
+		sw_output_parms.its = mvip_slot;
+		sw_output_parms.ost = sm_channel_info_parms.ist;
+		sw_output_parms.ots = sm_channel_info_parms.its;
+
+		sw_output_parms.mode = CONNECT_MODE;
+		rc = sw_set_output(sm_channel_info_parms.card, &sw_output_parms);
+		if(rc)
+		{
+			slog(Slog::levelError) << name << ": sw_set_output failed (" << rc << ")" << endl;
+			return false;
+		}
+#endif
+	}
 	/*
 	 * Set the MVIP stream and slot being used (if MVIP is not used,
 	 * -1 is passed back), so that the thread which called this function
@@ -510,11 +626,11 @@ void AculabTrunk::disconnectFromDSP()
 {
 	struct output_parms sw_output_parms;
 	struct sm_channel_info_parms sm_channel_info_parms;
-	
+
 	sm_channel_info_parms.channel=dspChannel;
 	sm_channel_info(&sm_channel_info_parms);
 	port=port;
-	
+
 	/*
 	 * Check if prosody channel and network port are on the
 	 * same card.
@@ -537,7 +653,11 @@ void AculabTrunk::disconnectFromDSP()
 		}
 		else {
 			/* if prosody channel is on a BR card, use switch API calls */
+#ifdef MVIP_BASED
 			sw_output_parms.ost=mvip_stream+8;
+#else
+			sw_output_parms.ost = mvip_stream + MVIP_FD_STREAMS;
+#endif
 			sw_output_parms.ots=mvip_slot;
 			sw_output_parms.mode=DISABLE_MODE;
 			sw_set_output(sm_channel_info_parms.card,&sw_output_parms);
