@@ -29,6 +29,7 @@ bool CapiTrunk::sleepHandler(TrunkEvent *event)
 
 	timeout_t wakeup;
 	char buffer[16];
+	char *cp;
 
 	switch(event->id) {
 	case TRUNK_ENTER_STATE:
@@ -71,9 +72,11 @@ bool CapiTrunk::sleepHandler(TrunkEvent *event)
                 handler = &CapiTrunk::stepHandler;
                 return true;
         case TRUNK_CHILD_START:
-                setSymbol(data.sleep.save, 64);
-                setSymbol(data.sleep.save,
-                        event->parm.trunk->getSymbol(SYM_GID));
+		cp = event->parm.trunk->getSymbol(SYM_GID);
+		if(cp)
+			cp = strchr(cp, '-');
+		if(cp)
+			setVariable(data.sleep.save, 16, cp);
                 trunkSignal(TRUNK_SIGNAL_STEP);
                 handler = &CapiTrunk::stepHandler;
                 return true;
@@ -119,8 +122,7 @@ bool CapiTrunk::sleepHandler(TrunkEvent *event)
 				if(value)
 				{
 					*(value++) = 0;
-					setSymbol(cp, getSymbolSize());
-					setSymbol(cp, value);
+					setVariable(cp, 0, value);
 				}
 				cp = strtok_r(NULL, "&", &sp);
 			}
