@@ -707,8 +707,7 @@ bool CapiTrunk::nActHandler(_cmsg *cmsg)
 		} else if (record_flag) {
 			CapiBuffer *buf = new CapiBuffer(cmsg->DataLength);
 
-			memcpy(buf->data, (void *) cmsg->Data32,
-			       cmsg->DataLength);
+			memcpy(buf->data, cmsg->Data, cmsg->DataLength);
 
 			record_queue.queueTail(buf);
 			thread->post();
