@@ -89,6 +89,8 @@ Keydata("/bayonne/dialogic")
 	load(defkeys);
 }
 
+#include "possibilite.c"
+
 bool DialogicConfig::getRouting(void)
 {
 	const char *rtflag = getLast("routing");
@@ -105,6 +107,8 @@ Driver(), Thread(keythreads.priService() - 1)
 	static Script::Define keywords[] = {
 		{"join", (Method)&DialogicTrunk::scrJoin, &ScriptCommand::chkHasArgs},
 		{"wait", (Method)&DialogicTrunk::scrWait, &ScriptCommand::chkHasArgs},
+		{"reroute", (Method)&DialogicTrunk::scrReroute, &ScriptCommand::chkHasArgs},
+		{"conference", (Method)&DialogicTrunk::scrConference, &ScriptCommand::chkHasArgs},
 		{NULL, NULL, NULL}};
 
 	char d4xname[32];
@@ -115,11 +119,12 @@ Driver(), Thread(keythreads.priService() - 1)
 	int dticard = 1, voxcard = 1, pricard = 0, isdncard = 0;
 	int dti = 0;
 	int voxcount, vox, dticount = 0;
-	int id = 0;
+	int id = 0, queue;
 	int chdev, tsdev;
 	int maxslots = getTimeslots();
 	interface_t iface = NT_E1;
 	char *span;
+	const char *cp = getLast("queue");
 	CT_DEVINFO ctinfo;
 
 	level = 0;
@@ -171,6 +176,13 @@ Driver(), Thread(keythreads.priService() - 1)
 		memset(isdn, 0, sizeof(DialogicISDN *) * isdn_count);
 	}
 
+	if(cp)
+	{
+		queue = atoi(cp);
+		sr_setparm(SRL_DEVICE, SR_QSIZEID, &queue);
+		slog(Slog::levelDebug) << "dialogic: event queue depthset to " << queue << endl;
+	}
+
 #ifndef	SIGNAL
 	slog(Slog::levelDebug) << "dialogic: setting polled mode" << endl;
 	mode = SR_POLLMODE;
@@ -415,7 +427,8 @@ Driver(), Thread(keythreads.priService() - 1)
 	
 	ScriptCommand::load(keywords);
 				
-	slog(Slog::levelInfo) << "Dialogic driver loaded; capacity=" << id << endl;
+	slog(Slog::levelInfo) << "Dialogic driver loaded; capacity=" << id << 
+		"; maxconference=" << DialogicDCB::detect() << endl;
 	port_used = id;
 }
 
@@ -578,11 +591,22 @@ void DialogicDriver::run(void)
 		case CCEV_CONGESTION:
 		case CCEV_DISCONNECTED:
 		case CCEV_DIVERTED:
+			datap = sr_getevtdatap();
+			cc_GetLineDev(&evdev, datap);
+			break;
 		case CCEV_FACILITY:
-#ifdef	CCEV_FACILITYACK
-		case CCEV_FACILITYACK:
-		case CCEV_FACILITYREJ:
-#endif
+			datap = sr_getevtdatap();
+			cc_GetLineDev(&evdev, datap);
+			break;
+		case CCEV_FACILITY_ACK:
+			datap = sr_getevtdatap();
+			cc_GetLineDev(&evdev, datap);
+			break;
+		case CCEV_FACILITY_REJ:
+			datap = sr_getevtdatap();
+			cc_GetLineDev(&evdev, datap);
+			break;
+
 		case CCEV_HOLDCALL:
 		case CCEV_L2FRAME:
 		case CCEV_L2NOBFFR:
@@ -677,6 +701,40 @@ void DialogicDriver::run(void)
                        	trunk->putEvent(&event);
 			break;
 		case CCEV_OFFERED:
+			// Modification to extract Commande Possibilite
+			datap = sr_getevtdatap();
+			IE_BLK ie_blk;
+			char bufPos[64 + 1];
+			char NAS[20 + 1];
+			char CTX[20 + 1];
+			char APPELANT[20 + 1];
+			char RETOUR[20 + 1];
+			int i, lenPos;
+
+			strcpy(trunk->callednasposs, "UNKNOWN");
+			strcpy(trunk->callingnasposs, "UNKNOWN");
+			strcpy(trunk->contextposs, "NONE");
+			strcpy(trunk->retourposs, "NONE");
+			if (cc_GetSigInfo((char *) &ie_blk, U_IES, datap) < 0) {
+				slog(Slog::levelError) << "Error on GetSigInfo" << endl;
+			} else {
+				slog(Slog::levelDebug) << "SigInfo length= " << ie_blk.length << endl;
+				for (i = 0 ; i < ie_blk.length; i++)
+					slog(Slog::levelDebug) << hex << (int) ie_blk.data[i] << endl;
+
+				/* Extract "Command Possibilit" from the ie block data */
+				if ((lenPos = extraitPossibilite(ie_blk.data, bufPos, ie_blk.length)) > 0) {
+					if (extraitInfosPossibilite(bufPos, NAS, APPELANT, CTX, RETOUR, lenPos) >= 0)  {
+						strcpy(trunk->callednasposs, NAS);
+						strcpy(trunk->callingnasposs, APPELANT);
+						strcpy(trunk->contextposs, CTX);
+						strcpy(trunk->retourposs, RETOUR);
+					}
+				} else slog(Slog::levelDebug) << "No commande Possibilite" << endl;
+			}
+
+			// End of modification
+
 			if(cc_GetCRN(&tmpcrn, datap) < 0)
 				trunk->putISDNError("cc_GetCRN");
 			else
@@ -699,9 +757,6 @@ void DialogicDriver::run(void)
 			if(cc_GetInfoElem(evdev, &ie_blk) < 0)
 				trunk->putISDNError("cc_GetSignInfo ");
 			slog(Slog::levelDebug) << "IE recv: ";
-			for(i=0; i < ie_blk.length; i++)
-				slog() << hex << (int)ie_blk.data[i] <<  " ";
-			slog() << endl;
 			break;
 		case CCEV_DROPCALL:
 			trunk->releaseISDN();
@@ -956,135 +1011,191 @@ void DialogicDriver::run(void)
 			}
 			break;
 		case DTEV_T1ERRC:
+
+			slog(Slog::levelError) << "dialogic: DTEV_T1ERRC"  << endl;
 			// this needs fixing, we should find the right span involved
 			if(isdn_count)
 			{
+				/* Etoile Diese : is it always the first span ? */
 				isdn = getSpan(1);
-				iface = isdn->getInterface();
+
+				/* Etoile Diese : don't know why, but here, iface = isdn->getInterface();  crashes. */
+				/* Let's hard-set iface to NT_E1 */
+				iface = NT_E1; 
 			}
-			else
+			else {
 				iface = trunk->interface;
+			}
+
 			alarmp = (unsigned char *)sr_getevtdatap();
-			switch(iface)
-			{
-			case NT_T1:
-				switch(*alarmp)
+
+			/* In case of sr_getevtdatap would not provide anything, let's test it */
+			if (alarmp) {
+				switch(iface)
 				{
-				case DTT1_RLOS:
-					slog(Slog::levelError) << "dialogic: remote loss of sync alarm" << endl;
-					break;
-				case DTT1_LOS:
-					slog(Slog::levelError) << "dialogic: local loss of sync alarm" << endl;
-					break;
-				case DTT1_DPM:
-					slog(Slog::levelError) << "dialogic: performance monitor alarm" << endl;
-					break;
-				case DTT1_RED:
-					slog(Slog::levelError) << "dialogic: T1 red alarm" << endl;
-					break;
-				case DTT1_BPVS:
-					slog(Slog::levelError) << "dialogic: bipolar violation alarm" << endl;
-					break;
-				case DTT1_ECS:
-					slog(Slog::levelError) << "dialogic: error count saturation alarm" << endl;
-					break;
-				case DTT1_RYEL:
-					slog(Slog::levelError) << "dialogic: T1 yellow alarm" << endl;
-					break;
-				case DTT1_RCL:
-					slog(Slog::levelError) << "dialogic: carrier lost alarm" << endl;
-					break;
-				case DTT1_FERR:
-					slog(Slog::levelError) << "dialogic: frame bit error alarm" << endl;
-					break;
-				case DTT1_B8ZSD:
-					slog(Slog::levelError) << "dialogic: bipolar eight zero substitution alarm" << endl;
-					break;
-				case DTT1_RBL:
-					slog(Slog::levelError) << "dialogic: T1 blue alarm" << endl;
-					break;
-				case DTT1_OOF:
-					slog(Slog::levelError) << "dialogic: out of frame error alarm" << endl;
-					break;
-				case DTT1_B8ZSDOK:
-					// sample alarm handling code from Intel suggests we should
-					// disable this alarm once we receive it once...
-					param = DTSP_B8ZS;
-					dt_setparm(evdev, DTG_CODESUPR, (void *)&param);
-					dt_setevtmsk(evdev, DTG_T1ERREVT, DTEC_B8ZSD, DTA_SUBMSK);
-				case DTT1_RLOSOK:
-				case DTT1_LOSOK:
-				case DTT1_REDOK:
-				case DTT1_BPVSOK:
-				case DTT1_ECSOK:
-				case DTT1_RYELOK:
-				case DTT1_RCLOK:
-				case DTT1_FERROK:
-				case DTT1_RBLOK:
-				case DTT1_OOFOK:
-					slog(Slog::levelError) << "dialogic: alarm recovery" << endl;
-					break;
-				}
-				break;
-			case NT_E1:
-				switch(*alarmp)
-				{
-				case DTE1_LOS:
-					slog(Slog::levelError) << "dialogic: local loss of sync alarm" << endl;
-					break;
-				case DTE1_DPM:
-					slog(Slog::levelError) << "dialogic: performance monitor alarm" << endl;
-					break;
-				case DTE1_BPVS:
-					slog(Slog::levelError) << "dialogic: bipolar violation alarm" << endl;
-					break;
-				case DTE1_ECS:
-					slog(Slog::levelError) << "dialogic: error count saturation alarm" << endl;
-					break;
-				case DTE1_RRA:
-					slog(Slog::levelError) << "dialogic: received remote alarm" << endl;
-					break;
-				case DTE1_FSERR:
-					slog(Slog::levelError) << "dialogic: frame sync alarm" << endl;
-					break;
-				case DTE1_RUA1:
-					slog(Slog::levelError) << "dialogic: unframed all 1's alarm" << endl;
-					break;
-				case DTE1_RLOS:
-					slog(Slog::levelError) << "dialogic: remote loss of sync alarm" << endl;
-					break;
-				case DTE1_MFSERR:
-					slog(Slog::levelError) << "dialogic: multiframe sync alarm" << endl;
-					break;
-				case DTE1_RSA1:
-					slog(Slog::levelError) << "dialogic: signalling all 1's alarm" << endl;
-					break;
-				case DTE1_RDMA:
-					slog(Slog::levelError) << "dialogic: distant multiframe alarm" << endl;
-					break;
-				case DTE1_CECS:
-					slog(Slog::levelError) << "dialogic: CRC4 error count alarm" << endl;
-					break;
-				case DTE1_LOSOK:
-				case DTE1_DPMOK:
-				case DTE1_BPVSOK:
-				case DTE1_ECSOK:
-				case DTE1_RRAOK:
-				case DTE1_FSERROK:
-				case DTE1_RUA1OK:
-				case DTE1_RLOSOK:
-				case DTE1_MFSERROK:
-				case DTE1_RSA1OK:
-				case DTE1_RDMAOK:
-				case DTE1_CECSOK:
-					slog(Slog::levelError) << "dialogic: alarm recovery" << endl;
+				case NT_T1:
+					switch(*alarmp)
+					{
+					case DTT1_RLOS:
+						slog(Slog::levelError) << "dialogic: remote loss of sync alarm" << endl;
+						break;
+					case DTT1_LOS:
+						slog(Slog::levelError) << "dialogic: local loss of sync alarm" << endl;
+						break;
+					case DTT1_DPM:
+						slog(Slog::levelError) << "dialogic: performance monitor alarm" << endl;
+						break;
+					case DTT1_RED:
+						slog(Slog::levelError) << "dialogic: T1 red alarm" << endl;
+						break;
+					case DTT1_BPVS:
+						slog(Slog::levelError) << "dialogic: bipolar violation alarm" << endl;
+						break;
+					case DTT1_ECS:
+						slog(Slog::levelError) << "dialogic: error count saturation alarm" << endl;
+						break;
+					case DTT1_RYEL:
+						slog(Slog::levelError) << "dialogic: T1 yellow alarm" << endl;
+						break;
+					case DTT1_RCL:
+						slog(Slog::levelError) << "dialogic: carrier lost alarm" << endl;
+						break;
+					case DTT1_FERR:
+						slog(Slog::levelError) << "dialogic: frame bit error alarm" << endl;
+						break;
+					case DTT1_B8ZSD:
+						slog(Slog::levelError) << "dialogic: bipolar eight zero substitution alarm" << endl;
+						break;
+					case DTT1_RBL:
+						slog(Slog::levelError) << "dialogic: T1 blue alarm" << endl;
+						break;
+					case DTT1_OOF:
+						slog(Slog::levelError) << "dialogic: out of frame error alarm" << endl;
+						break;
+					case DTT1_B8ZSDOK:
+						// sample alarm handling code from Intel suggests we should
+						// disable this alarm once we receive it once...
+						param = DTSP_B8ZS;
+						dt_setparm(evdev, DTG_CODESUPR, (void *)&param);
+						dt_setevtmsk(evdev, DTG_T1ERREVT, DTEC_B8ZSD, DTA_SUBMSK);
+					case DTT1_RLOSOK:
+					case DTT1_LOSOK:
+					case DTT1_REDOK:
+					case DTT1_BPVSOK:
+					case DTT1_ECSOK:
+					case DTT1_RYELOK:
+					case DTT1_RCLOK:
+					case DTT1_FERROK:
+					case DTT1_RBLOK:
+					case DTT1_OOFOK:
+						slog(Slog::levelError) << "dialogic: alarm recovery" << endl;
+						break;
+					}
 					break;
+				case NT_E1:
+					switch(*alarmp)
+					{
+					case DTE1_LOS:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_LOS : local loss of sync alarm" << endl;
+						break;
+					case DTE1_DPM:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_DPM : performance monitor alarm" << endl;
+						break;
+					case DTE1_BPVS:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_BPVS : bipolar violation alarm" << endl;
+						break;
+					case DTE1_ECS:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_ECS : error count saturation alarm" << endl;
+						break;
+					case DTE1_RRA:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RRA : received remote alarm" << endl;
+						break;
+					case DTE1_FSERR:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_FSERR : frame sync alarm" << endl;
+						break;
+					case DTE1_RUA1:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RUA1 : unframed all 1's alarm" << endl;
+						break;
+					case DTE1_RLOS:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RLOS : remote loss of sync alarm" << endl;
+						break;
+					case DTE1_MFSERR:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_MFSERR : multiframe sync alarm" << endl;
+						break;
+					case DTE1_RSA1:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RSA1 : signalling all 1's alarm" << endl;
+						break;
+					case DTE1_RDMA:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RDMA : distant multiframe alarm" << endl;
+						break;
+					case DTE1_CECS:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_CECS : CRC4 error count alarm" << endl;
+						break;
+
+					case DTE1_LOSOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_LOSOK : Received loss of signal recovered" << endl;
+						break;
+					case DTE1_DPMOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_DPMOK : Driver perf. monitor failure recovered" << endl;
+						break;
+					case DTE1_BPVSOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_BPVSOK : Bipolar violation cnt saturation recovered" << endl;
+						break;
+					case DTE1_ECSOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_ECSOK : Error count saturation recovered" << endl;
+						break;
+					case DTE1_RRAOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RRAOK : Received remote alarm recovered" << endl;
+						break;
+					case DTE1_FSERROK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_FSERROK : Received frame sync error recovered" << endl;
+						break;
+					case DTE1_RUA1OK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RUA1OK : Received unframed all 1's recovered" << endl;
+						break;
+					case DTE1_RLOSOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RLOSOK : Received loss of sync recovered" << endl;
+						break;
+					case DTE1_MFSERROK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_MFSERROK : Received multi frame sync error recovered" << endl;
+						break;
+					case DTE1_RSA1OK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RSA1OK : Received signalling all 1's recovered" << endl;
+						break;
+					case DTE1_RDMAOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_RDMAOK : Recved. dist. multi-frame alarm recovered" << endl;
+						break;
+					case DTE1_CECSOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_CECSOK : CRC4 error count saturation recovered" << endl;
+						break;
+					case DTE1_REDOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_REDOK : Red alarm recovered" << endl;
+						break;
+					case DTE1_LOOPBACK_CFAOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_LOOPBACK_CFAOK : Diagnostic mode on line trunk recovered" << endl;
+						break;
+					case DTE1_DCHAN_CFAOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_DCHAN_CFAOK :  D channel out of service recovered" << endl;
+						break;
+					case DTE1_CRC_CFAOK:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - DTE1_CRC_CFAOK : Timeslot 16 CRC failure recovered" << endl;
+						break;
+					default:
+						slog(Slog::levelError) << "dialogic: DTEV_T1ERRC - default : " << endl;
+						break;
+					}
 				}
+			} else {
+				slog(Slog::levelError) <<  "DTEV_T1ERRC : alarmp : no data." << endl;
 			}
 			break;
 		case DX_ATOMIC_ERR:
 			trunk->postError(trunk->chdev, "atomic error ");
 			break;
+		case CCEV_FACILITY:
+			event.id = TRUNK_CALL_FACILITY;
+			trunk->putEvent(&event);
+			break;
 		default:
 			slog(Slog::levelError) << "UNHANDLED EVENT: " << evtype << endl;
 		}
