@@ -221,10 +221,11 @@ Driver(), Thread(keythreads.priService() - 1)
 	int boardtype;
 	int card = 0;
 	int vox = 0;
-	int id = 0;
+	int id = 0, queue;
 	unsigned i, s, portidx;
 	GC_CCLIB_STATUSALL cclibstatus;
 	GC_INFO gcinfo;
+	const char *cp = getLast("queue");
 
 	level = 0;
 	status = DialogicTrunk::status;
@@ -248,6 +249,13 @@ Driver(), Thread(keythreads.priService() - 1)
 	memset(devmap, 0, sizeof(devices_t) * port_count);
 	memset(spanmaps, 0, sizeof(DialogicISDN *) * getSpanCount());
 
+	if(cp)
+	{
+		queue = atoi(cp);
+		sr_setparm(SRL_DEVICE, SR_QSIZEID, &queue);
+		slog(Slog::levelDebug) << "dialogic: event queue depth set to " << queue << endl;
+	}
+
 #ifndef	SIGNAL
 	slog(Slog::levelDebug) << "globalcall: setting polled mode" << endl;
 	mode = SR_POLLMODE;
@@ -536,6 +544,7 @@ void DialogicDriver::run(void)
 		case GCEV_FATALERROR:
 		case GCEV_TASKFAIL:
 			event.id = TRUNK_CALL_FAILURE;
+			EventError(&metaevent);
 			trunk->postEvent(&event);
 			break;
 		case GCEV_BLOCKED:
@@ -740,7 +749,18 @@ void DialogicDriver::run(void)
 			case TDX_PLAYTONE:
 			case TDX_DIAL:
 			case TDX_CALLP:
-				event.id = TRUNK_TONE_IDLE;
+				switch(event.id)
+				{
+					case TRUNK_CPA_NOANSWER:
+					case TRUNK_CPA_NODIALTONE:
+					case TRUNK_CPA_NORINGBACK:
+					case TRUNK_CPA_FAILURE:
+					case TRUNK_CPA_BUSYTONE:
+						break;
+
+					default:
+						event.id = TRUNK_TONE_IDLE;
+				}
 				break;
 			case TDX_PLAY:
 			case TDX_RECORD:
@@ -753,6 +773,8 @@ void DialogicDriver::run(void)
 			event.id = TRUNK_TIMER_EXPIRED;
 			trunk->putEvent(&event);
 			break;
+		case CCEV_SETCHANSTATE:
+			break;
 		default:
 			slog(Slog::levelDebug) << "globalcall: unhandled event 0x" << hex << evtype << endl;
 		}
