@@ -64,26 +64,43 @@ PikaConfig::PikaConfig() :
 Keydata("/bayonne/pika")
 {
 	static KEYDEF defkeys[] = {
-	{"buffers", "2"},
+	{"buffers", "4"},
 	{"ringdebounce", "60"},
+	{"members", "16"},
+	{"groups", "8"},
 	{NULL, NULL}};
 
+	if(chkHosting(NULL))
+		Load("/hosting/pika");
+
 	Load(defkeys);
 }
 
 PikaDriver::PikaDriver() :
 Driver()
 {
+	static SCRKEYWORDS keywords[] = {
+		{"join", (scriptmethod_t)&PikaTrunk::scrJoin, &ScriptCommand::chkHasArgs},
+		{"wait", (scriptmethod_t)&PikaTrunk::scrWait, &ScriptCommand::chkHasArgs},
+		{"enter", (scriptmethod_t)&PikaTrunk::scrConf, &ScriptCommand::chkHasArgs},
+		{"leave", (scriptmethod_t)&PikaTrunk::scrLeave, &ScriptCommand::chkNoArgs},
+		{"detect", (scriptmethod_t)&PikaTrunk::scrDetect, &ScriptCommand::chkHasArgs},
+		{"duplex", (scriptmethod_t)&PikaTrunk::scrDuplex, &ScriptCommand::chkHasArgs},
+		{NULL, NULL, NULL}};
+
 	int id;
 	unsigned trkcount, dspcount;
 	unsigned good = 0;
+	unsigned confgroups = getMixerGroups();
 
 	keyflag = false;
 	dsps = NULL;
 	ports = NULL;
 	groups = NULL;
 	dsp_count = 0;
+	conf_count = 0;
 	port_count = 0;
+	mixer_count = 0;
 	card_count = PK_CTI_NumberOfBoards();
 
 	for(id = 0; id < card_count; ++id)
@@ -139,16 +156,21 @@ Driver()
 	// dsp devices
 	dsps = new trunkdsp_t[card_count];
 	ports = new PikaTrunk *[port_count];
+	conf = new PikaConf *[dsp_count * confgroups];
+	mixer = new PikaMixer *[dsp_count];
 	groups = new TrunkGroup *[port_count];
 	memset(dsps, 0, sizeof(trunkdsp_t) * card_count);
 	memset(ports, 0, sizeof(PikaTrunk *) * port_count);
 	memset(groups, 0, sizeof(TrunkGroup *) * port_count);
+	memset(conf, 0, sizeof(PikaConf *) * dsp_count);
 	status = PikaTrunk::status = new char[port_count];
 	memset(status, ' ', port_count);
 
 	slog(SLOG_INFO) << "Pika driver loaded " << good << " of " << card_count
 		<< " cards; trunks=" << port_count << ", dsps=" << dsp_count << endl;
 
+
+	ScriptCommand::Load(keywords);
 	ccxx_sleep(4000);
 }
 
@@ -157,13 +179,19 @@ PikaDriver::~PikaDriver()
 	Stop();
 	
 	if(dsps)
-		delete dsps;
+		delete[] dsps;
 
 	if(ports)
-		delete ports;
+		delete[] ports;
 
 	if(groups)
-		delete groups;
+		delete[] groups;
+
+	if(conf)
+		delete[] conf;
+
+	if(mixer)
+		delete[] mixer;
 
 	delete[] PikaTrunk::status;
 	PikaTrunk::status = NULL;
@@ -171,10 +199,18 @@ PikaDriver::~PikaDriver()
 
 int PikaDriver::Start(void)
 {
+	static TPatternState StateTable[3][2] = {
+		{{1000 - 100, 2000 + 100}, {3000 - 100, 4000 + 100}},  //ringback
+		{{ 500 - 100,  500 + 100}, { 500 - 100,  500 + 100}},  //busy
+		{{ 250 -  50,  250 +  50}, { 250 -  50,  250 +  50}}}; //fail
+
 	TResourceHandle hRes;
+	int pat = 0;
 	int count = 0;
 	int card = 0;
+	int grp, grpmax = getMixerGroups();
 	int dsp, dsp_count;
+	TResourceHandle hGrp, hDsp;
 
 	if(active)
 	{
@@ -191,15 +227,37 @@ int PikaDriver::Start(void)
 
 		while(dsp < dsp_count)
 		{
-			dsps[card].dsp[dsp] = PK_DSP_GetDeviceHandle(card, dsp);
-			if(dsps[card].dsp[dsp])
-				slog(SLOG_DEBUG) << "dsp: activated; card=" << card << ", device=" << dsp << endl;
-			else
+			hDsp = dsps[card].dsp[dsp] = PK_DSP_GetDeviceHandle(card, dsp);
+			if(!hDsp)
+			{
 				slog(SLOG_ERROR) << "dsp: failed; card=" << card << " device=" << dsp << endl;
+				++dsp;
+				continue;
+			}
+			slog(SLOG_DEBUG) << "dsp: activated; card=" << card << ", device=" << dsp << endl;
+			++dsp;
+			mixer[mixer_count] = new PikaMixer(hDsp, conf_count);
+			grp = 0;
+			while(grp < grpmax)
+			{
+				hGrp = PK_VC_CreateGroup(hDsp);
+				if(!hGrp)
+					break;
+				conf[conf_count++] = new PikaConf(mixer[mixer_count], hGrp);
+				++grp;
+			}
+			if(grp)
+				mixer[mixer_count++]->setGroups(grp);
 			++dsp;
 		}
 	}
 
+	cpTable = PK_PAT_CreateTable(3);
+	PK_PAT_MatchLastOnTimeout(cpTable, PK_FALSE);
+	while(pat < 3)
+		PK_PAT_AddPattern(cpTable, PK_ATD_CALL_PROGRESS_TONE,
+			StateTable[pat++], 2);
+	
 	card = 0;
 	while(card < card_count && count < port_count)
 	{
@@ -224,6 +282,7 @@ int PikaDriver::Start(void)
 		++card;
 	}
 
+	slog(SLOG_INFO) << "conf: initialized " << conf_count << " groups  from " << mixer_count << " mixers" << endl;
 
 	active = true;
 	return count;
@@ -248,6 +307,18 @@ void PikaDriver::Stop(void)
 			delete (ports[id]);
 	}
 
+	for(id = 0; id < conf_count; ++id)
+	{
+		if(conf[id])
+			delete (conf[id]);
+	}
+
+	for(id = 0; id < mixer_count; ++id)
+	{
+		if(mixer[id])
+			delete (mixer[id]);
+	}
+
 	for(id = 0; id < card_count; ++id)
 		PK_DSP_StopDriver(id);
 
@@ -257,11 +328,28 @@ void PikaDriver::Stop(void)
 		PK_MVIP_StopDriver(id);
 	}
 
+	PK_PAT_DeleteTable(cpTable);
 	memset(dsps, 0, sizeof(trunkdsp_t) * card_count);
 	memset(ports, 0, sizeof(PikaTrunk *) * port_count);
 	slog(SLOG_INFO) << "driver stopping..." << endl;
 }
 
+Conference *PikaDriver::getConference(int id)
+{
+	if(id < 0 || id >= conf_count)
+		return NULL;
+
+	return (Conference *)conf[id];
+}
+
+Mixer *PikaDriver::getMixer(int id)
+{
+	if(id < 0 || id >= mixer_count)
+		return NULL;
+
+	return (Mixer *)mixer[id];
+}
+
 Trunk *PikaDriver::getTrunkPort(int id)
 {
 	if(id < 0 || id >= port_count)
