@@ -26,6 +26,7 @@ bool VPBTrunk::busyHandler(TrunkEvent *event)
 	switch(event->id)
 	{
 	case TRUNK_ENTER_STATE:
+		flags.cid = false;
 		synctimer = exittimer = 0;
 		enterState("busy");
 		switch(flags.trunk)
@@ -39,7 +40,8 @@ bool VPBTrunk::busyHandler(TrunkEvent *event)
 		flags.trunk = TRUNK_MODE_UNAVAILABLE;
 		status[id] = '*';
 		endTimer();
-		disjoin();
+		disjoin(TRUNK_STOP_DISCONNECT);
+		clearRinging();
 		if(thread)
 		{
 			setTimer(thread->stop());
@@ -54,6 +56,8 @@ bool VPBTrunk::busyHandler(TrunkEvent *event)
 		flags.dsp = DSP_MODE_INACTIVE;
 		flags.offhook = true;
 		return true;
+	case TRUNK_STATION_FLASH:
+	case TRUNK_STATION_ONHOOK:
 	case TRUNK_STOP_DISCONNECT:
 	case TRUNK_LINE_WINK:
 	case TRUNK_CPA_DIALTONE:
