@@ -25,6 +25,9 @@ bool VPBTrunk::flashoffHandler(TrunkEvent *event)
 {
 	switch(event->id)
 	{
+	case TRUNK_DTMF_KEYUP:
+	case TRUNK_STOP_DISCONNECT:
+		return true;
 	case TRUNK_CPA_DIALTONE:
 		endTimer();
 		setSymbol(SYM_TONE, "dial");
@@ -66,6 +69,9 @@ bool VPBTrunk::flashonHandler(TrunkEvent *event)
 {
 	switch(event->id)
 	{
+	case TRUNK_DTMF_KEYUP:
+	case TRUNK_STOP_DISCONNECT:
+		return true;
 	case TRUNK_TIMER_EXPIRED:
 		handler = &VPBTrunk::flashoffHandler;
 		return true;
