@@ -221,6 +221,7 @@ bool Fifo::setSymbol(char **argv)
 {
 	Script::Symbol *sym;
 	Trunk *trunk = getTrunk(argv[1]);
+	bool rts;
 	if(!trunk)
 		return false;
 
@@ -228,21 +229,9 @@ bool Fifo::setSymbol(char **argv)
 		return false;
 
 	trunk->enterMutex();
-	sym = trunk->getLocal(argv[2], trunk->getSymbolSize());
-	if(sym)
-	{
-		if(!sym->flags.readonly)
-		{
-			snprintf(sym->data, sym->flags.size + 1, "%s",  argv[3]);
-			if(sym->flags.commit)
-				trunk->commit(sym);
-		}
-		
-	}
+	rts = trunk->setVariable(argv[2], 0, argv[3]);
 	trunk->leaveMutex();
-//	trunk->setSymbol(argv[2], trunk->getSymbolSize());
-//	trunk->setSymbol(argv[2], argv[3]);
-	return true;
+	return rts;
 }
 
 bool Fifo::deleteId(char **argv)
@@ -870,7 +859,7 @@ bool Fifo::command(const char *cmd, ostream *fd)
 	buffer[sizeof(buffer) - 1] = 0;
 
 	enterMutex();
-	slog(Slog::levelDebug) << "fifo: cmd=" << cmd << endl;
+	slog(Slog::levelDebug) << "fifo: cmd=" << buffer << endl;
 	if(strstr(buffer, token))
 	{
 		ptr = buffer;
